declare filename "SOShat.dsp"; declare name "SOShat"; //import("stdfaust.lib");

// Port from SuperCollider (SC) to Faust of SynthDef \SOShat in 
// <SuperCollider>/examples/demonstrations/DrumSynths.scd
// e.g., /usr/local/share/SuperCollider/examples/demonstrations/DrumSynths.scd
// on Linux
// based on a Sound-on-Sound 'synth secrets' tutorial:
// http://www.soundonsound.com/sos/Jun02/articles/synthsecrets0602.asp

//https://github.com/josmithiii/faust-jos/tree/master/percussion

PI = 3.1415926535897932385;
SR = 44100;
MAX_SAW_ORDER = 4;

decay(n,x) = x - (x>0.0)/n;
release(n) = + ~ decay(n);
envgate(dur,trigger) = trigger : release(int(dur*float(SR))) : >(0.0);

adsr(a,d,s,r,t) = env ~ (_,_) : (!,_) // the 2 'state' signals are fed back
with {
    env (p2,y) =
        (t>0) & (p2|(y>=1)),          // p2 = decay-sustain phase
        (y + p1*u - (p2&(y>s))*v*y - p3*w*y)	// y  = envelop signal
	*((p3==0)|(y>=eps)) // cut off tails to prevent denormals
    with {
		p1 = (p2==0) & (t>0) & (y<1);         // p1 = attack phase
		p3 = (t<=0) & (y>0);                  // p3 = release phase
		// #samples in attack, decay, release, must be >0
		na = SR*a+(a==0.0); nd = SR*d+(d==0.0); nr = SR*r+(r==0.0);
		// correct zero sustain level
		z = s+(s==0.0)*db2linear(-60);
		// attack, decay and (-60dB) release rates
		u = 1/na; v = 1-pow(z, 1/nd); w = 1-1/pow(z*db2linear(60), 1/nr);
		// values below this threshold are considered zero in the release phase
		eps = db2linear(-120);
    };
};

impulse = 1-1';
oscrs(f) = impulse : nlf2(f,1) : _,!; // sine

nlf2(f,r,x) = ((_<:_,_),(_<:_,_) : (*(s),*(c),*(c),*(0-s)) :> (*(r),+(x))) ~ cross
with {
  th = 2*PI*f/SR;
  c = cos(th);
  s = sin(th);
  cross = _,_ <: !,_,_,!;
};

perc(att,rel,trigger) = adsr(att,0,1.0,rel,envgate(att,trigger));
pmosc(carfreq,modfreq,index) = oscrs(carfreq + (index*modfreq)
			     * oscrs(modfreq));

line(start,end,dur,trigger) = trigger : release(int(dur*float(SR))) 
		    : *(start-end)+end;
		    
lpf(freq) = lowpass(3,freq);
hpf(freq) = highpass(3,freq);

lowpass(N,fc) = lowpass0_highpass1(0,N,fc);
highpass(N,fc) = lowpass0_highpass1(1,N,fc);

lowpass0_highpass1(s,N,fc) = lphpr(s,N,N,fc)
with {
  lphpr(s,0,N,fc) = _;
  lphpr(s,1,N,fc) = tf1s(s,1-s,1,2*PI*fc);
  lphpr(s,O,N,fc) = lphpr(s,(O-2),N,fc) : tf2s(s,0,1-s,a1s,1,w1) with {
    parity = N % 2;
    S = (O-parity)/2; // current section number
    a1s = -2*cos((PI)*-1 + (1-parity)*PI/(2*N) + (S-1+parity)*PI/N);
    w1 = 2*PI*fc;
  };
};

tf1s(b1,b0,a0,w1) = tf1(b0d,b1d,a1d)
with {
  c   = 1/tan(w1*0.5/SR); // bilinear-transform scale-factor
  d   = a0 + c;
  b1d = (b0 - b1*c) / d;
  b0d = (b0 + b1*c) / d;
  a1d = (a0 - c) / d;
};

tf2s(b2,b1,b0,a1,a0,w1) = tf2(b0d,b1d,b2d,a1d,a2d)
with {
  c   = 1/tan(w1*0.5/SR); // bilinear-transform scale-factor
  csq = c*c;
  d   = a0 + a1 * c + csq;
  b0d = (b0 + b1 * c + b2 * csq)/d;
  b1d = 2 * (b0 - b2 * csq)/d;
  b2d = (b0 - b1 * c + b2 * csq)/d;
  a1d = 2 * (a0 - csq)/d;
  a2d = (a0 - a1*c + csq)/d;
};

tf1(b0,b1,a1) = _ <: *(b0), (mem : *(b1)) :> + ~ *(0-a1);
tf2(b0,b1,b2,a1,a2) = iir((b0,b1,b2),(a1,a2));

bpf(freq, rq) = tf2(1.0,0,-1.0,-2.0*R*cos(theta),R^2) with {
  theta = 2.0*PI*freq/SR;
  R = -PI*freq*rq/SR;
};

tf2(b0,b1,b2,a1,a2) = iir((b0,b1,b2),(a1,a2));

iir(bv,av) = sub ~ fir(av) : fir(bv);
sub(x,y) = y-x;


square = squareN(2);
squareN(N,freq) = pulsetrainN(N,freq,0.5);

lf_sawpos(freq) = frac ~ +(freq/SR);
frac(n) = n - floor(n);

sawN(N,freq) = saw1l : poly(Nc) : D(Nc-1) : gate(Nc-1)
with {
  Nc = max(1,min(N,MAX_SAW_ORDER));
  clippedFreq = max(20.0,abs(freq)); // use lf_sawpos(freq) for LFOs (freq < 20 Hz)
  saw1l = 2*lf_sawpos(clippedFreq) - 1; // zero-mean, amplitude +/- 1
  // Also note the availability of lf_sawpos_phase above.
  poly(1,x) = x;
  poly(2,x) = x*x;
  poly(3,x) = x*x*x - x;
  poly(4,x) = x*x*(x*x - 2.0);
  poly(5,x) = x*(7.0/3 + x*x*(-10.0/3.0 + x*x));
  poly(6,x) = x*x*(7.0 + x*x*(-5.0 + x*x));
  p0n = float(SR)/clippedFreq; // period in samples
  diff1(x) = (x - x')/(2.0/p0n);
  diff(N) = seq(n,N,diff1); // N diff1s in series
  factorial(0) = 1;
  factorial(i) = i * factorial(i-1);
  D(0) = _;
  D(i) = diff(i)/factorial(i+1);
  gate(N) = *(1@(N)); // delayed step for blanking startup glitch
};

pulsetrainN(N,freq,duty) = diffdel(sawN(N,freqC),del) with {
    diffdel(x,del) = x-x@int(del)*(1-frac(del))-x@(int(del)+1)*frac(del);
 DELPWR2 = 2048;
 delmax = DELPWR2-1;
 SRmax = 96000.0;
 fmin = SRmax / float(2.0*delmax);
 freqC = max(freq,fmin);
 period = (float(SR) / freqC);
 ddel = duty * period;
 del = max(0,min(delmax,ddel));
};

fir((b0,bv)) = _ <: *(b0), R(1,bv) :> _ with {
	R(n,(bn,bv)) = (@(n):*(bn)), R(n+1,bv);
	R(n, bn)     = (@(n):*(bn));};
fir(b0) = *(b0);

pulse0p5(freq) = square(freq); // phase 0, duty cycle 0.5

gate = checkbox("gate");
db2linear(l) = pow(10, l/20.0);
smooth(s) = *(1.0 - s) : + ~ *(s);

ampdb  = hslider("gain",-20,-60,40,0.1);
amp = ampdb : smooth(0.999) : db2linear;

sustain  = hslider("sustain",1,0.0001,10,0.00001);
freq  = hslider("freq",6000,10,10000,0.00001);

//freq = 6000; // orig value
// sustain = 0.1; // orig value

trigger = gate > gate';

root_cymbal_square = pulse0p5(freq);
root_cymbal = root_cymbal_square <: 
    pmosc(freq*1.34, 310.0/1.3) + 
    pmosc(freq*2.405, 26.0/0.5) +
    pmosc(freq*3.09, 11.0/3.4) + 
    pmosc(freq*1.309, 0.72772);
initial_bpf_contour = line(1, 9000, 0.1, trigger);
initial_env = perc(0.005, 0.1, trigger);
initial_bpf = root_cymbal : bpf(initial_bpf_contour,1.0) * initial_env;

body_env = sqrt(perc(0.005, sustain, trigger)); 
body_hpf = root_cymbal : hpf(line(1, SR/100, sustain, trigger)) * body_env;
cymbal_mix = (initial_bpf + body_hpf) * amp;

process = cymbal_mix;