declare filename "duckDelaySt.dsp"; declare name "duckDelaySt"; declare id   "duckDelaySt";
declare name "Duck Delay St";
declare category "Echo / Delay";

//------------------------------------
//Description:
//The delayed signal added to output dependent of input signal amplitude. 
//If the input signal is high. The delayed signall turned off, and vise versa.
//The switching controlled by envelope follower 
//(parameters: "attack", "release", and main - "amount", what is controls envelope follower influence).
//
//Parameters description:
//time - de.delay time in milliseconds 
//feedback - de.delay feedback
//pingpong - changes feedback character
//coloration - LP/HP coloration filter
//attack, release - envelope follower time in seconds controls
//amount dB - envelope follower influence
//effect - amplitude of effect signal in mix
//------------------------------------

//import("stdfaust.lib");

SR = 44100;
EPSILON = 1.192092896e-07;
PI = 3.1415926535897932385;

db2linear(l) = pow(10, l/20.0);

tau = if(clipCond, 0.0, exp(-1.0/(tauCenterClipped*SR))) with {
    clipCond = abs(tau)<EPSILON;
    tauCenterClipped = if(clipCond, 1.0, tau); // 1.0 can be any nonzero value (not used)
};

if(cond,then,else) = select2(cond,else,then);

//Constrols
p_time = hslider("time", 500, 1, 2000, 1):smooth(tau2pole(0.1));
p_feedback = hslider("feedback", 0, 0, 1, 0.05);
p_pingpong = hslider("pingpong", 0, 0, 1, 0.05);
coloration = hslider("coloration", 0, -1, 1, 0.05);

p_attack_time = hslider("attack", 0.1, 0.05, 0.5, 0.05);
p_release_time = hslider("release", 0.1, 0.05, 2, 0.05);
p_amount = hslider("amount", 0.5, 0,56, 0.05):db2linear;

p_effect = hslider("effect", 0, -16, +4, 0.1) : db2linear : smooth(0.999);

//Consts
c_fdelay_max_len = 393216;
c_channels_sw_time = 0.1;

//Funcs
X = (_,_)<:(!,_,_,!);
get_delay_length(x) = x*SR:_*0.001;
coloration_filter(coloration) = _<:(lowshelf(5,(1 - coloration)*12,440),
	highshelf(5,coloration*12,880)):>_*db2linear(-15);

pp_delay(time,fb_coef,pp_fb_coef) = _,_*(1 - pp_fb_coef):
	(_,X,_:(X:(pp_fb_delay(time, fb_coef,pp_fb_coef))),
	(X:(pp_fb_delay(time, fb_coef,pp_fb_coef))):_,_)~X
	:>_,_
with {
	pp_fb_delay(time,fb_coef,pp_fb_coef) = _+_*pp_fb_coef:
	(_+_:fdelay(c_fdelay_max_len,get_delay_length(p_time)))~_*fb_coef;
};

amp_follower_ud(att,rel) = amp_follower(rel) : smooth(tau2pole(att));

amp_follower(rel) = abs : env with {
 p = tau2pole(rel);
 env(x) = x * (1.0 - p) : (+ : max(x,_)) ~ *(p);
};

switcher(att, rel, amount) = amp_follower_ud(att,rel):
	_*amount:_>1:(1 - _):
	smooth(tau2pole(c_channels_sw_time));

tau2pole(tau) = if(clipCond, 0.0, exp(-1.0/(tauCenterClipped*float(SR)))) with {
 clipCond = abs(tau)<EPSILON;
 tauCenterClipped = if(clipCond, 1.0, tau); // 1.0 can be any nonzero value (not used)
};

smooth(s) = *(1.0 - s) : + ~ *(s);

fdelay(n,d,x) = delay(n+1,int(d),x)*(1 - frac(d)) + delay(n+1,int(d)+1,x)*frac(d);
delay(n,d,x) = x @ min(n, max(0,d));
frac(n) = n - floor(n);

highshelf(N,Lpi,fx) = filterbank(N,(fx)) : *(db2linear(Lpi)), _ :> _;
lowshelf(N,L0,fx) = filterbank(N,(fx)) : _, *(db2linear(L0)) :> _;

filterbank(O,lfreqs) = analyzer(O,lfreqs) : delayeq(nb) with
{
   nb = count(lfreqs);
   fc(n) = take(n, lfreqs);
   ap(n) = highpass_plus_lowpass(O,fc(n));
   delayeq(1) = _,_; // par(i,0,...) does not fly
   delayeq(nb) = par(i,nb-1,apchain(nb-1-i)),_,_;
   apchain(0) = _;
   apchain(i) =  ap(i) : apchain(i-1);
};

highpass(N,fc) = lowpass0_highpass1(1,N,fc);
lowpass(N,fc) = lowpass0_highpass1(0,N,fc);

lowpass0_highpass1(s,N,fc) = lphpr(s,N,N,fc)
with {
  lphpr(s,0,N,fc) = _;
  lphpr(s,1,N,fc) = tf1s(s,1-s,1,2*PI*fc);
  lphpr(s,O,N,fc) = lphpr(s,(O-2),N,fc) : tf2s(s,0,1-s,a1s,1,w1) with {
    parity = N % 2;
    S = (O-parity)/2; // current section number
    a1s = -2*cos((PI)*-1 + (1-parity)*PI/(2*N) + (S-1+parity)*PI/N);
    w1 = 2*PI*fc;
  };
};

tf1s(b1,b0,a0,w1) = tf1(b0d,b1d,a1d)
with {
  c   = 1/tan(w1*0.5/SR); // bilinear-transform scale-factor
  d   = a0 + c;
  b1d = (b0 - b1*c) / d;
  b0d = (b0 + b1*c) / d;
  a1d = (a0 - c) / d;
};

tf1(b0,b1,a1) = _ <: *(b0), (mem : *(b1)) :> + ~ *(0-a1);

count((xs, xxs)) = 1 + count(xxs);
count(xx) = 1;

take(1, (xs, xxs))  = xs;
take(1, xs)         = xs;
take(nn, (xs, xxs)) = take(nn-1, xxs);

analyzer(O,lfreqs) = _ <: bsplit(nb) with
{
   nb = count(lfreqs);
   fc(n) = take(n, lfreqs);
   lp(n) = lowpass(O,fc(n));
   hp(n) = highpass(O,fc(n));
   bsplit(0) = _;
   bsplit(i) = hp(i), (lp(i) <: bsplit(i-1));
};

tf2s(b2,b1,b0,a1,a0,w1) = tf2(b0d,b1d,b2d,a1d,a2d)
with {
  c   = 1/tan(w1*0.5/SR); // bilinear-transform scale-factor
  csq = c*c;
  d   = a0 + a1 * c + csq;
  b0d = (b0 + b1 * c + b2 * csq)/d;
  b1d = 2 * (b0 - b2 * csq)/d;
  b2d = (b0 - b1 * c + b2 * csq)/d;
  a1d = 2 * (a0 - csq)/d;
  a2d = (a0 - a1*c + csq)/d;
};

tf2(b0,b1,b2,a1,a2) = iir((b0,b1,b2),(a1,a2));

iir(bv,av) = sub ~ fir(av) : fir(bv);
sub(x,y) = y-x;

fir((b0,bv)) = _ <: *(b0), R(1,bv) :> _ with {
	R(n,(bn,bv)) = (@(n):*(bn)), R(n+1,bv);
	R(n, bn)     = (@(n):*(bn)); };
fir(b0) = *(b0);

process = (_<:_,_,_),(_<:_,_,_):
	_,
	switcher(p_attack_time, p_release_time, p_amount),
	(pp_delay(p_time,p_feedback*(1 - p_pingpong),p_pingpong*p_feedback):
	coloration_filter(coloration),coloration_filter(coloration)),
	switcher(p_attack_time, p_release_time, p_amount),
	_:
	_,_*_,_*_,_:(_,_*p_effect:>_),(_*p_effect,_:>_);