declare filename "moog_vcf.dsp"; declare name "moog_vcf"; //-------------------------(ve.)moog_vcf---------------------------
// Moog "Voltage Controlled Filter" (VCF) in "analog" form. Moog VCF
// implemented using the same logical block diagram as the classic
// analog circuit.  As such, it neglects the one-sample delay associated
// with the feedback path around the four one-poles.
// This extra delay alters the response, especially at high frequencies
// (see reference [1] for details).
// See moog_vcf_2b below for a more accurate implementation.
//
// #### Usage
//
// 
// moog_vcf(res,fr)
// 
// Where:
//
// * res: normalized amount of corner-resonance between 0 and 1 
// (0 is no resonance, 1 is maximum)
// * fr: corner-resonance frequency in Hz (less than SR/6.3 or so)
//
// #### References
// * <https://ccrma.stanford.edu/~stilti/papers/moogvcf.pdf>
// * <https://ccrma.stanford.edu/~jos/pasp/vegf.html>
//------------------------------------------------------------

declare moog_vcf author "Julius O. Smith III";
declare moog_vcf copyright "Copyright (C) 2003-2019 by Julius O. Smith III <jos@ccrma.stanford.edu>";
declare moog_vcf license "MIT-style STK-4.3 license";

import("stdfaust.lib");

moog_vcf(res,fr) = (+ : seq(i,4,fi.pole(p)) : *(unitygain(p))) ~ *(mk)
with {
     p = 1.0 - fr * 2.0 * ma.PI / ma.SR; // good approximation for fr << SR
     unitygain(p) = pow(1.0-p,4.0); // one-pole unity-gain scaling
     mk = -4.0*max(0,min(res,0.999999)); // need mk > -4 for stability
};

res = hslider("Res", 0, 0., 1., 0.001);
fr = hslider("Freq", 1000, 10, 10000, 1):si.smoo;

process = _ : moog_vcf(res,fr) :_;