declare filename "peak_dynamical_compression.dsp"; declare name "peak_dynamical_compression"; // dynamical compression based on peak

ma = library("maths.lib");
ba = library("basics.lib");
si = library ("signals.lib");

EPSILON = 2.2204460492503131e-016;

// sampling period
sp = 1/ma.SR;

// Converts a number of samples to a duration in seconds.
samp2sec = /(ma.SR);

// 1-pole coeff. for 60-dB decay in a desired time
rt60(x) =   .001 ,(sp , max(x, .001) : /) : pow;

// infinitely fast attack and adjustable release
peak_env(window, in) = abs(in) : max ~ *(window : rt60);

tau2pole(tau) = ba.if(clipCond, 0.0, exp(-1.0/(tauCenterClipped*float(ma.SR))))
with {
		clipCond = abs(tau)<EPSILON;
 		tauCenterClipped = ba.if(clipCond, 1.0, tau); // 1.0 can be any nonzero value (not used)
};

smooth(s) = *(1.0 - s) : + ~ *(s);

dyn_comp_peak(window, curve, in) = in, cucurve : * , envel, cucurve
with {
		cucurve = (1-min(envel, 1) : pow(curve));
		envel = peak_env(window, in) : smooth(tau2pole(0.004));
};

gain = hslider("gain", 1., 0.0001, 10.0, 0.00001):si.smoo;
curve = hslider("curve", 1.0, 0.00001, 10000.0, 0.00001):si.smoo;
//release = hslider("release", 1.0, 0.00001, 10000.0, 0.00001)/1000;
window = hslider("window", 882, 0, 48000, 1) : samp2sec;

process = _ : dyn_comp_peak(window, curve) : _ *gain , _, _;