declare filename "peak_dynamical_normalisation.dsp"; declare name "peak_dynamical_normalisation"; // dynamical normalisation based on peak

//ma = library("maths.lib");
//ba = library("basics.lib");
//si = library ("signals.lib");

EPSILON = 2.2204460492503131e-016;
SR = 44100;

// Converts a number of samples to a duration in seconds.
samp2sec = /(SR);

// sampling period
sp = 1/SR;

// NAN-safe divider
div(x1, x2) = if(x2 : ==(0), 0, (x1 ,x2 : /));

// 1-pole coeff. for 60-dB decay in a desired time
rt60(x) = .001 ,(sp , max(x, .001) : /) : pow;

// infinitely fast attack and adjustable release
//peak_env(window, in) = abs(in) : max ~ *(window : rt60);
peak_env(window, in) = abs(in) : max ~ *(window : rt60);

tau2pole(tau) = if(clipCond, 0.0, exp(-1.0/(tauCenterClipped*float(SR))))
with {
		clipCond = abs(tau)<EPSILON;
 		tauCenterClipped = if(clipCond, 1.0, tau); // 1.0 can be any nonzero value (not used)
};

smooth(s) = *(1.0 - s) : + ~ *(s);

// dynamical normalisation based on peak
dyn_norm_peak(window, target, interp, input) = input * ratio : dcblocker, ratio, envel
with {
		envel = peak_env(window, input) : smooth(tau2pole(0.004));
		ratio = target, envel : div  : smooth(tau2pole(interp));
};

dcblocker = zero(1) : pole(0.995);
zero(z) = _ <: _,mem : _,*(z) : -;
pole(p) = + ~ *(p);
if(cond,then,else) = select2(cond,else,then);

target = hslider("target", 1.0, 0.00001, 10.0, 0.00001);
window = hslider("window", 882, 0, 48000, 1) : samp2sec;
interp = hslider("interp", 480, 1, 48000, 1) : samp2sec;

process = _ : dyn_norm_peak(window, target, interp) : _,_,_ ;