declare filename "tubeTremolo.dsp"; declare name "tubeTremolo"; declare name "Tube Tremolo";
declare category "Modulation";

/*
** Model of a vactrol tremolo unit by "transmogrify"
** c.f. http://sourceforge.net/apps/phpbb/guitarix/viewtopic.php?f=7&t=44&p=233&hilit=transmogrifox#p233
** http://transmogrifox.webs.com/vactrol.m
*/

//import("stdfaust.lib");
//si = library("signals.lib");
//fi = library("filters.lib");

//import("guitarix.lib");
//import("redeye.lib");

/* vactrol model */

SR = 44100;
PI = 3.1415926535897932385;

R1 = 2700;
Ra = 1e6;
Rb = 300;

b = exp(log(Ra)/log(Rb)) - exp(1);
dTC = 0.06;
minTC = log(0.005/dTC);

cds = ((_ <: _,_),_ : _+(1-alpha)*_) ~ (_<:*(alpha)) with {
    iSR = 1/SR;
    dRC = dTC * exp(*(minTC));
    alpha = 1 - iSR / (dRC + iSR);
};

sub(x,y) = y-x;
iir(bv,av) = sub ~ fir(av) : fir(bv);

fir((b0,bv)) = _ <: *(b0), R(1,bv) :> _ with {
	R(n,(bn,bv)) = (@(n):*(bn)), R(n+1,bv);
	R(n, bn)     = (@(n):*(bn));              };
fir(b0) = *(b0);

impulse = 1-1';

oscs(f) = (*(-1) : sint(wn) : sintp(wn,impulse)) ~ _
with {
  wn = 2*PI*f/SR; // approximate
  // wn = 2*sin(PI*f/SR); // exact
  sint(x) = *(x) : + ~ _ ; // frequency-scaled integrator
  sintp(x,y) = *(x) : +(y): + ~ _ ; // same + state input
};

smooth(s) = *(1.0 - s) : + ~ *(s);

if(cond,then,else) = select2(cond,else,then);

// 12AT7
input12at7 = pre : iir((b0/a0,b1/a0),(a1/a0)) with {
    LogPot(a, x) = if(a, (exp(a * x) - 1) / (exp(a) - 1), x);
    Inverted(b, x) = if(b, 1 - x, x);
    s = 0.993;
    fs = float(SR);
    pre = _;
    Drive = vslider("Drive[name:Drive]", 0.5, 0, 1, 0.01) : Inverted(0) : LogPot(1) : smooth(s);        
        b0 = -0.000564641478707507*Drive*fs;
        b1 = 0.000564641478707507*Drive*fs;
        a0 = 2.08239024527461e-5*fs + 0.00045268226818868;
        a1 = -2.08239024527461e-5*fs + 0.00045268226818868;
};

output12au7 = pre : iir((b0/a0,b1/a0),(a1/a0)) with {
    LogPot(a, x) = if(a, (exp(a * x) - 1) / (exp(a) - 1), x);
    Inverted(b, x) = if(b, 1 - x, x);
    s = 0.993;
    fs = float(SR);
    pre = _;
    Gain = hslider("gain", 0.5, 0, 1, 0.01) : Inverted(1) : LogPot(1) : smooth(s);
        b0 = 0.000257077328090757*Gain*fs - 0.000257077328090757*fs;
        b1 = -0.000257077328090757*Gain*fs + 0.000257077328090757*fs;
        a0 = 2.08237464507809e-5*fs + 0.000460170362516827;
        a1 = -2.08237464507809e-5*fs + 0.000460170362516827;
};

vactrol = pow(_,1.9) : cds : *(b) + exp(1) : exp(log(Ra)/log) : R1/(_ + R1);

/* os.triangle oscillator (not bandlimited, frequency is approximate) */

trianglewave(freq) = _ ~ (_ <: _ + hyst) : /(periodsamps) with {
    if(c,t,e) = select2(c,e,t);
    hyst(x) = if(_ > 0, 2 * (x < periodsamps) - 1, 1 - 2 * (x > 0)) ~ _;
    periodsamps = int(SR / (2*float(freq)));
};

/* tremolo unit, using os.triangle or sine oscillator as lfo */

tremolo(freq, depth) = lfo * depth + 1 - depth : vactrol with {
    sine(freq) = (oscs(freq) + 1) / 2 : max(0); // max(0) because of numerical inaccuracy
    SINE=checkbox("sine[enum:os.triangle|sine]");
    lfo = select2(SINE, trianglewave(freq), sine(freq));
};

process = input12at7:*(0.1):*(tremolo(hslider("speed",5,0.1,14,0.1),hslider("depth",0.5,0,1,0.01))):output12au7;